! Module: omdrifthist_mod
! Authors: 
!          Tim Harries, Jon Rainger
!          Chris Brindle    
!
! Description: Produces PPS products OM postscript tracking history
! 
! Comments: 
! 
! $Id: module.f90,v 1.4 1997/12/18 16:01:10 gvacanti Exp $
! 
! History:
!   
!      29 Jan 98:    Converted to run under DAL
!                    Two page format added
!                    Hardcopy option added
!                    Roll drift now plotted
!      18 Feb 98:    Added more text info to output
!                    Now checks HK for BPE binning flag
! 
!      27 Apr 98:    Removed datestamp call (TJH)
!      14 May 98:    Checks pgbeg stat 
!                    Output filename length extended
!
!      20 May 99:    Removed dependancy on periodic housekeeping file (JFR).  
!                    Added checks on input DATAMODE (JFR).
!                    General tidy (JFR).
!
!      30 May 2000   Added info files to task
!
!      2nd June 2000 Added tidy up for allocated memory (AMK)
!      25th July 2000 Bug removed - same file opened twice.
!
!      15th December 2000 (CB) - extra information added to tracking history
!      plot (SPR 2125
!      8th of January 2002 (VNY) - dependencies file has been updated for higher version of ssclib (2.0)
!********************************************************************************)
module omdrifthist_mod
  use cal
  use dal
  use param  
 
  implicit none 

  private 

  public omdrifthist

  type point                                                  ! data type for points
     real(single) :: x
     real(single) :: y
  end type point
  !************************************
  ! The following structure will store
  ! exposure information from a fits
  ! header file
  !************************************
  type ExposureInformation
      character(80)   :: observationID
      character(80)   :: exposureID
      character(80)   :: object
      character(10)   :: filter
      integer(int32) :: frameTime
      logical             :: binbpe
  end type ExposureInformation 
             


contains 

!---------------------------------------------------------------------
  subroutine omdrifthist
!---------------------------------------------------------------------
    type(point), allocatable        :: history(:)             ! The tracking history
    type(point), allocatable        :: increments(:)          ! Incremental tracking points
    type(point)                     :: minPlot, maxPlot       ! Bottom left and top right to t.h. plot
    type (DataSetT)                 :: checkDataSetH
    type(TableT)                    :: table    
    character(255)              :: set                    ! Tracking history filename
    character(255)              :: plotfile               ! Output postscript filename
    character(100)              :: cString                ! Plot string
    character(1024)             :: device                 ! The device name.

    integer, parameter              :: nhist=50               ! number of bins for histogra
    integer(int32)             :: i                      ! loop counters
    integer(int32)             :: pgstat                 ! Pgplot status
    integer(int32)             :: nFrames                ! The number of good frames in the tracking history.
    integer(int32)             :: nBad                   ! Number of bad excursions
    integer(int32)             :: indexHist              ! Index of histogram
    integer(int32)             :: nOutside               ! Number of points outside nominal tracking radius.
    integer(int32)             :: pgbeg                  ! PGPLOT       

    real(single)               :: trackRadius            ! The TBD nominal tracking radius (a task parameter)
    real, allocatable               :: incDrift(:)            ! Magnitudes of incremental tracking vectors
    real(single)               :: maxR,minR              ! Maximum/minimum extent of plot
    real(single)               :: r                      ! A radius
    real(single)               :: rPixel                 ! This radius in pixels
    real(single)               :: binHist                ! Size of histogram bins
    real(single)               :: xHistogram(nhist)      ! Histogram
    real(single)               :: nxHistogram(nhist)     ! X histogram
    real(single)               :: nyHistogram(nhist)     ! Y histogram
    real(single)               :: vx1,vx2,vy1,vy2        ! Size of pgplot vport
    real(single)               :: xRange                 ! X range
    real(single), allocatable  :: realFrame(:)           ! Real values of frame number
    real(single)               :: range                  ! Range for plot
    real(single), parameter    :: pi=3.141592564         ! PI
    real(single), allocatable  :: degroll(:)             ! Calculated roll in degrees.
    real(single), pointer      ::  x(:) ,y(:), roll(:)   ! Pointer for drifts and roll
    real(single)               :: meanDrift              ! Mean drift value.
    
    logical(BOOL)              :: pageOne, pageTwo
    logical(Bool)              :: hardcopy               ! Hardcopy required?
    logical(Bool)              :: badArrow               ! Does a bad arrow need to be plotted?
    real(double)               :: lambda
    real(double)               :: plateScale
    integer(int32)             :: filterID   
    type(ExposureInformation)       :: exposureInfo 
    real(single)               :: xStart, yStart
     
!
!//////////////////////////////////////////////////////////////////////
!
! AMK
!
    integer              :: istat   ! status flag
!
!//////////////////////////////////////////////////////////////////////
!

! Get input parameters.


    hardcopy = booleanParameter("hardcopy")   
    set = TRIM(stringParameter("set"))
    trackradius = realParameter("trackradius")
    
    ! Check if zero drift.


    checkDataSetH = dataSet(set, read)

    if (hasBlock(checkDataSetH, "ZERODRIFT")) then      
        write (errstr,  '(a)') "No PPS Tracking History Products available" 
        call warning("ZERODRIFT")

        call release(checkDataSetH) 
    else
        call release(checkDataSetH) 
! Check which pages are required by the user for plotting.

        do i = 1, parametercount("pages")

            if ( intParameter("pages", i - 1) .eq. 1) then
                pageOne = .true.
            end if

            if ( intParameter("pages", i - 1) .eq. 2) then
                pageTwo = .true.
            end if

        end do


        if (hardcopy) then
            plotFile = stringParameter("plotfile")
        end if
    
        ! Read in the OM Tracking History Data Auxiliary file.


        call readHist(set, x, y, roll, nFrames, nBad, exposureInfo)

        !****************************************************************
        ! Issue a warning if there are no good tracking-history frames
        !****************************************************************
        if (nFrames .eq. 0) then
            call warning("No good tracking-history frames found- can't produce any plots")
            !******************************************************
            ! Add "ZERODRIFT" block to say tracking file no good
            !****************************************************** 
            checkDataSetH = dataSet(set, modify)

            table = addTable(checkDataSetH, "ZERODRIFT", 1)
            call release(table)
            call release(checkDataSetH)
            !*********************************
            ! Stop the program here
            !*********************************
            stop
        end if
        




        !****************************
        ! Get the plate-scale
        !****************************

        filterID = filterIdToInt(trim(exposureInfo%filter))
        call CAL_setState(OM, filterId = filterId)
        call CAL_omGetPlateScale(lambda, plateScale)

        trackradius = trackradius / plateScale

! Allocate arrays

        allocate(history(1 : nFrames))
        allocate(degroll(1 : nFrames))
        allocate(realFrame(1 : nFrames))
        allocate(incDrift(1 : nFrames))                 
    
        do i = 1, nFrames
            history(i)%x = x(i)
            history(i)%y = y(i)
        end do
    
! Convert nominal tracking radius to pixels.

!    rPixel = 2.*trackradius
        rPixel = trackradius    
        if (exposureInfo%binbpe) then
            rPixel = rPixel / 2.0
        end if
    
! release tracking history data.

! now find the maximum and minimum x and y values for the t.h. plot

        minPlot%x = 1.0e20
        maxPlot%x = -1.0e20
        minPlot%y = 1.0e20
        maxPlot%y = -1.0e20

        do i = 1 , nFrames
            minPlot%x = min(history(i)%x, minPlot%x)
            minPlot%y = min(history(i)%y, minPlot%y)
            maxPlot%x = max(history(i)%x, maxPlot%x)
            maxPlot%y = max(history(i)%y, maxPlot%y)
       
        end do
        xStart = history(1)%x
        yStart = history(1)%y 

! find the maximum absolute value and leave a little space about edge of plot.

        maxR = max(maxPlot%x, abs(minPlot%x), maxPlot%y, abs(minPlot%y))
        maxR = maxR * 1.05

! Now set up a histogram for the x and y values of the tracking drift
    
        binHist = 2.0 * maxR / real(nhist)                                      ! binHist is the size of the bins

        do i = 1 , nhist
            xHistogram(i) = maxR * (2.0 * real(i - 1) / real(nhist - 1) - 1.0)          ! set up the x array
        end do
    
        nxHistogram = 0.0                                                 ! initialize the histograms
        nyHistogram = 0.0


        do i = 1 , nFrames
            indexHist = history(i)%x / binHist+nhist / 2 + 1
            nxHistogram(indexHist) = nxHistogram(indexHist) + 1.0
            indexHist = history(i)%y / binHist+nhist / 2 + 1
            nyHistogram(indexHist) = nyHistogram(indexHist) + 1.0
        end do


! Now draw the plot. This is done using pgplot.
! if hardcopy = F then we ask the user for a device name, otherwise we
! write a postscript file with the filename plotfile.

        if (.not.hardcopy) then
            pgstat = pgbeg(0,'?', 1, 1)
        else
            device = trim(plotfile) // "/VPS"
            pgstat =  pgbeg(0, trim(device), 1, 1)
        end if

        if (pgstat .ne. 1) then
            call error("badDevice","Cannot open pgplot device")
        end if


        call pgscf(2)            ! fancier font

    if (pageOne) then

! The lower left area of the page

       call pgvport(0.1, 0.85, 0.05, 0.8)   ! draw the box, was 0.1, 0.7, 0.0, 0.7
       call pgwnad(-maxR, maxR, -maxR, maxR)
       call pgbox('bcnst', 0.0, 0, 'bcnst', 0.0, 0)
       call pgsch(1.25)
       call pgslw(3)
       call pglab('\fi x (pixels)','\fi y (pixels)',' ')

! Loop through the t.h. drawing arrows. If the x,y position is outside the XXXX TBD
! radius then it is plotted as a dashed line, and the number of bad points is
! incremented.

       call pgsch(0.5)

       nOutside = 0
       incDrift(1) = modulus(history(1))
       do i = 2 , nFrames
          
          incDrift(i) = modulus(subpoints(history(i), history(i - 1)))           ! set the incremental magnitude here
          
          badArrow = .false.
          r = sqrt((xStart - history(i)%x) ** 2 + (yStart - history(i)%y) ** 2)
          if (r .gt. rPixel) badArrow = .true.                           ! is this tracking drift outside `nominal' radius?
          
          if (badArrow) then
             nOutside = nOutside + 1
             
             
             call pgsls(2)                                              ! if so plot as dashed line
          else
             call pgsls(1)
          endif
          
          
          call pgarro(history(i - 1)%x, history(i - 1)%y, history(i)%x, history(i)%y)    ! draw the arrow
          
       end do
       
       meanDrift = SUM(incDrift) / real(nFrames - 1)
       
       call pgsch(1.0)

! Draw a dotted circle demarking the good region

       call pgsfs(2)
       call pgsls(4)
       call pgcirc(xStart, yStart, rPixel)
       call pgwindow(0.0, 100.0, 0.0, 100.0)
       call pgmove(5.0, 95.0)
       call pgdraw(10.0, 95.0)
       call pgsls(1)
       call pgsch(0.6)
       call pgptext(12.0, 94.0, 0.0, 0.0,"Nominal tracking circle")
       call pgsch(1.0)

! Draw histograms top and right of the vector plot...

       r = -1.e20                                                        ! find the largest value and leave a small gap at top of plot
       do i = 1, nhist
          r = max(nxHistogram(i), r)
       enddo
       r = 1.05 * r
       
       call pgqvp(1, vx1, vx2, vy1, vy2)                                    ! Draw the histogram on top. The physical size of this
       call pgvsiz(vx1, vx2, vy2, vy2 + 1.0)                                  ! is one inch - *this doesn't have to be hardwired
       call pgwindow(-maxR, maxR, 0.00001, r)                              ! but would take some time to change*
       call pgbox('bc', 0.0, 0,'bcnst', 0.0, 0)
       call pghist(nFrames, history%x, -maxR, maxR, 50,1)
       call pglab(" ","\fi Freq"," ")
       call pgmtext('t', 1.3, 0.5, 0.5, "\fi Tracking history vector plot")
       call pgsls(1)

! Now for the histogram to the right - more difficult as it has to be done `by hand'
! since pghist only draws horizontally

       r =-1.e20                                                        ! find the range
       do i = 1, nhist
          r = max(nyHistogram(i), r)
       enddo
       r = 1.05 * r
       
       call pgvsiz(vx2, vx2+1.0, vy1, vy2)
       call pgwindow(0.00001, r, -maxR, maxR)
       call pgbox('bcnst', 0.0, 0, 'bc',0.0, 0)                               ! draw the box
       
       do i = 1 , nhist                                                 ! draw the histogram bars
          call pgmove(0.0, xHistogram(i) - binHist / 2.0)
          call pgdraw(nyHistogram(i), xHistogram(i) - binHist/2.0)
          call pgdraw(nyHistogram(i), xHistogram(i) + binHist/2.0)
          call pgdraw(0.0, xHistogram(i) + binHist / 2.0)
       enddo
       call pgsls(1)
       
       call pglab("\fi Freq"," ", " ")                                  ! label it


! Now put a label on the first page at the top left

       call pgvport(0.0, 1.0, 0.0, 1.0)
       call pgwindow(0.0, 100.0, 0.0, 100.0)
       call pgsch(1.25)
       call pgslw(3)
 
       call  PlotExposureDetails(exposureInfo, real(plateScale), nFrames, nBad)


    endif
    call PlotPageNumber(1)
    call PlotProgramVersionNo


! The next page was divided into four sections:

    if (pageTwo) then
    
       
       nOutside = 0
       incDrift(1) = modulus(history(1))
       do i = 2 , nFrames
          
          incDrift(i) = modulus(subpoints(history(i), history(i - 1)))           ! set the incremental magnitude here
          
          badArrow = .false.
          r = modulus(history(i))
          if (r .gt. rPixel) badArrow = .true.                           ! is this tracking drift outside `nominal' radius?
          
          if (badArrow) then
             nOutside = nOutside + 1
          end if
          
       end do

       meanDrift = SUM(incDrift) / real(nFrames - 1)
      
       call pgpage        ! the top left is filled with more labels
        
       call pgsvp(0.0, 1.0, 0.0, 1.0)
       call pgwindow(0.0, 100.0, 0.0, 100.0)

       call  PlotExposureDetails(exposureInfo, real(plateScale), nFrames, nBad)

       write(cString,'(a,f5.1,a,f5.1,a)') &
            'Percentage of excursions outside ',trackradius,' arc sec:',100.0 * (real(nOutside) / real(nFrames))," %"

       call pgmtxt("t", -10.0, 0.0, 0.0, trim(cString))

       write(cString,'(a,f5.1)') 'Mean drift per tracking frame (pixels): ', meanDrift
       call pgmtxt("t", -11.0, 0.0, 0.0, trim(cString))

       call pgsch(1.0)
       
! Calculate the incremental driftsize histogram


       maxR = -1.e20
       minR = 1.e20
       do i = 1 , nFrames
          maxR = max (maxR, incDrift(i))
          minR = min (minR, incDrift(i))
       end do
       xRange = maxR - minR
       maxR = maxR + 0.05 * xRange
       minR = minR - 0.05 * xRange


       binHist = (maxR-minR)/real(nhist)
       do i = 1 , nhist
          xHistogram(i) = minR + real(i - 1) * binHist
          nxHistogram(i) = 0.0
       end do
       
       do i = 1 , nFrames
          indexHist = 1 + ((incDrift(i) - minR) / (maxR - minR)) * nhist
          nxHistogram(indexHist) = nxHistogram(indexHist) + 1
       enddo
       
       r = -1.e20
       do i = 1, nhist
          r = max(nxHistogram(i), r)
       end do
       r = 1.05 * r
       
! Plot the incremental driftsize histogram
    
       call pgsvp(0.55, 0.95, 0.1, 0.55)
       call pgwindow(minR, maxR, 0.0001, r)
       call pgbox('bcnst',0.0, 0, 'bcnst', 0.0, 0)
       call pghist(nFrames, incDrift, minR, maxR, 50, 1)
       call pgmtxt("b", 2.3, 0.5, 0.5, "\fi Incremental drift size (pixels)")
       call pgmtxt("l", 2.0, 0.5, 0.5, "\fi Freq")
       call pgmtxt("t", 1.2, 0.5, 0.5, "\fi Drift size histogram")

! The next plot is a spot diagram for incremental drifts

       allocate(increments(1 : nFrames))

! Calculate the largest absolute incremental drift size in x and y

       maxR = -1.e30
       increments(1) = history(1)
       do i = 2, nFrames
          increments(i) = subpoints(history(i), history(i - 1))
          maxR = max(maxR,abs(increments(i)%x), abs(increments(i)%y))
       end do
       maxR = 1.05 * maxR

       !**********************************************
       ! Plot the incremental drift size spot diagram
       !**********************************************
       call pgsvp(0.55, 0.95, 0.58, 0.98)  
       call pgwnad(-maxR, maxR, -maxR, maxR)
       call pgbox('bcnst',0.0, 0, 'bcnst', 0.0, 0)
       write(*, *) nhist
      
       call pgpt(nFrames, increments%x, increments%y, -20)
       call pgmtext("b", 2.3, 0.5, 0.5,"\fi x (pixels)")
       call pgmtext("l", 2.3, 0.5, 0.5,"\fi y (pixels)")
       call pgmtext("t", 1.2, 0.5, 0.5,"\fi Incremental drift spot diagram")
       
! Find the maxinum and minimum roll values
       
       maxR=-1.e30
       minR = 1.e30
       do i = 1 ,nFrames
          realFrame(i) = real(i)
          degroll(i) = roll(i) * 180.0 / pi
          maxR = max(maxR, degroll(i))
          minR = min(minR, degroll(i))
       end do

       range = maxR - minR
        if (range .eq. 0.0) range = 1.0
       
        minR = minR - 0.05 * range
        maxR = maxR + 0.05 * range

! Plot the figure

        call pgsvp(0.08, 0.48, 0.1, 0.55)  
        call pgwindow(1.0, real(nFrames), minR, maxR)
        call pgbox('bcnst', 0.0, 0, 'bcnst', 0.0, 0)
     
        call pgbin(nFrames, realFrame, degroll, .true.)
        call pglab("\fi Frame number","\fi Roll drift (degrees)","\fi Roll drift plot")

    end if

        call PlotPageNumber(2)
        call PlotProgramVersionNo
       
        call pgend

    end if
   

!//////////////////////////////////////////////////////////
!
! AMK - Tidy up allocated memory
!
!/////////////////////////////////////////////////////////
   if(allocated(history)) then
       deallocate(history,stat=istat)
       if(istat .ne. 0) then
          call warning("Failed to release memory for history")
       end if
   end if
    
   if(allocated(degroll)) then
       deallocate(degroll,stat=istat)
       if(istat .ne. 0) then
          call warning("Failed to release memory for degroll")
       end if
   end if
 
   if(allocated(realFrame)) then
       deallocate(realFrame,stat=istat)
       if(istat .ne. 0) then
          call warning("Failed to release memory for realFrame")
       end if
   end if
 
   if(allocated(incDrift)) then
       deallocate(incDrift,stat=istat)
       if(istat .ne. 0) then
          call warning("Failed to release memory for incDrift")
       end if
   end if
 
   if(allocated(increments)) then
       deallocate(increments,stat=istat)
       if(istat .ne. 0) then
          call warning("Failed to release memory for increments")
       end if
   end if
 
end subroutine omdrifthist

!--------------------------------------------------------
! functions that act on type point:
!-------------------------------------------------------

real function modulus(a)
    type(point) a 

    modulus = sqrt(a%x * a%x + a%y * a%y)

end function modulus
!****************************************
! Function addpoints
!***************************************
function addpoints (a , b)
    type(point), intent(in) :: a
    type(point), intent(in) :: b
    type(point) addpoints
    addpoints%x = a%x + b%x
    addpoints%y = a%y + b%y
  end function addpoints
!****************************************
! Function subpoints
!***************************************
function subpoints (a , b)
    type(point), intent(in) :: a
    type(point), intent(in) :: b
    type(point) subpoints
    subpoints%x = a%x - b%x
    subpoints%y = a%y - b%y
end function subpoints

!********************************************************************
! Subroutine readHist
!********************************************************************
subroutine readHist(histFile, x, y, roll, nhist, nBad, exposureInfo)


    type(TableT) :: tableDataSet                      ! Handle on tracking history table
    type(ColumnT) :: pcolumn                          ! Handle on tracking history column.
    type(DataSetT) :: thxDataSetH                     ! Handle on tracking history file

    character*(*) ::  histFile                        ! tracking history filename.
   
    
    character(8) :: dataMode                      ! Input file data mode.

    integer(int32), pointer :: idx(:),idy(:)     ! Raw integer drifts pointers.
    integer(int32), pointer :: iroll(:)          ! Raw integer roll pointer.
    integer(int32), pointer :: iquality(:)       ! Raw integer quality pointer.
    integer(int32), pointer :: nggs(:)           ! Number of good guide stars pointer.
    integer(int32) :: nhist                      ! Number of tracking history frames.
    integer(int32) :: nBad                       ! Number of bad tracking frames.
    integer(int32) :: nrows                      ! Number of rows in tracking history.
                       
    integer :: i                                      ! Loop counter.
 
    real(single), pointer :: x(:),y(:),roll(:)   ! Calculated real roll and drifts
    real(single), allocatable :: quality (:)     ! Calculated real quality.
    real, parameter :: TBD = 10.                      ! Good quality definition.

    
    type(ExposureInformation), intent(out) :: exposureInfo

    integer       :: istat                            ! staus flag

    !***************************************************
    ! Get a handle on the tracking history data set.
    !**************************************************
    thxDataSetH = dataSet(histFile, read)
    
    !******************************************************
    ! Determine file type from datamode in primary header.
    !******************************************************

    dataMode = stringAttribute(thxDataSetH, "DATAMODE")



    if (dataMode .ne. "TRACKING") then

     
        !************************
        ! Incompatible file.
        !************************

        write (errstr, '(a, a, a, a)') "Invalid datamode ", dataMode, " in set ", histFile
        call fatal("badFileMode")
    end if

    !***********************************
    ! Get exposure specific attributes.
    !***********************************
    exposureInfo%observationID = stringAttribute(thxDataSetH,"OBS_ID")
    exposureInfo%exposureID = stringAttribute(thxDataSetH,"EXP_ID")
    exposureInfo%object = stringAttribute(thxDataSetH, "OBJECT")
    exposureInfo%filter = stringAttribute(thxDataSetH, "FILTER")
    exposureInfo%binbpe = booleanAttribute(thxDataSetH,"BINBPE")
    exposureInfo%frameTime = int32Attribute(thxDataSetH, "FRMTIME")
      

      
    ! Get a handle on the tracking history table.

    tableDataSet = table(thxDataSetH, 0)
  
    ! Get number of rows in table.
      
    nrows = numberOfRows(tableDataSet)   ! (No of tracking stars)
    
    !  Read data from columns.  
      
    pcolumn = column(tableDataSet, "DX", read)
    idx => int32data(pcolumn)
    pcolumn = column(tableDataSet, "DY", read)
    idy => int32data(pcolumn) 
    pcolumn = column(tableDataSet, "ROLL", read)
    iroll => int32data(pcolumn)
    pcolumn = column(tableDataSet, "QUALITY", read)
    iquality => int32data(pcolumn)
    pcolumn = column(tableDataSet, "NGGS", read)  
    nggs => int32data(pcolumn)
  
    ! allocate arrays for calculations.

    allocate(quality(0 : nrows - 1))
    allocate(x(1 : nrows))
    allocate(y(1 : nrows))
    allocate(roll(1 : nrows))
    
    ! Calculate real quality values.

    do i = 0 , nrows-1
        quality(i) = real(iquality(i)) / 1000000.0
        quality(i) = sqrt(quality(i) / (2.0 * real(nggs(i)) - 1.0))
    end do

! Calculate number of bad tracking frames, real drifts and rolls.
  
    nBad = 0
    nhist = 0

    do i = 0 , nrows - 1
        !********************************************************
        ! Check also that there are at least 3 good guide stars
        !******************************************************** 
        if (quality(i) .lt. TBD .and. nggs(i) .gt. 3) then             ! what constitutes a bad tracking soln?
            nhist = nhist + 1
            x(nhist) = real (idx(i)) / 1000.0
            y(nhist) = real (idy(i)) / 1000.0
            roll(nhist) = asin(real (iroll(i)) / 1000000.0)
        else
            nBad = nBad + 1
        end if
    end do



    call release(pcolumn)

    call release(tableDataSet)


    call release(thxDataSetH)
 


!
!--------------------------------------------------------------------------------
!
! AMK - Tidy up allocated memory
!

    if(allocated(quality)) then
        deallocate(quality,stat=istat)
        if(istat .ne. 0) then
            call warning("Failed to release memory for quality")
        end if
    end if  
   
end subroutine readHist

!***************************
! PlotProgramVersionNo
!***************************
subroutine PlotProgramVersionNo
  
    call pgsvp(0.0, 1.0, 0.0, 1.0)
    call pgsch(0.75)
    call pgslw(2)
      
    call pgmtxt("b", 0.0, 0.5, 0.5, "omdrifthist version 1.26")
end subroutine PlotProgramVersionNo
!*********************************
! PlotPageNumber
!*********************************
subroutine PlotPageNumber(no)
    integer, intent(in) :: no
    character(len=20) :: string

    call pgvport(0.0, 1.0, 0.0, 1.0)
     
    write(string, '(a, i1)') "Page ", no
    call pgsch(0.75)
    call pgslw(2)

    call pgmtxt("b", -0.5, 0.95, 1.0, trim(string))
end subroutine PlotPageNumber

!*******************************
! PlotExposureDetails
!*******************************
subroutine PlotExposureDetails(exposureInfo, plateScale, nFrames, nBad)
    type(ExposureInformation), intent(in) :: exposureInfo
    real, intent(in) ::  plateScale
    integer, intent(in) :: nFrames, nBad
    character(len=100) :: string

    write(string,'(a, a)') "OM tracking history plot- object ", trim(exposureInfo%object)
    call pgsch(1.25)
    call pgslw(3)
               
    call pgmtxt("t", -1.0, 0.5, 0.5, trim(string))

    call pgsch(0.75)
    call pgslw(2)  
   
    call pgmtxt("t", -3.0, 0.0, 0.0, "Observation ID: " // trim(exposureInfo%observationID(1 : 10)))
    call pgmtxt("t", -4.0, 0.0, 0.0, "Exposure ID: " // trim(exposureInfo%exposureID))

    write(string,'(a, a, a, f6.4, a)') "Filter ", trim(exposureInfo%filter), &
                   " Plate-scale=", plateScale, " arcsecs/pixel"   

    call pgmtxt("t", -5.0, 0.0, 0.0, trim(string))
  
    write(string,'(a, f6.1, a)') "Tracking frame time: ", & 
                  real(exposureInfo%frameTime) / 1024.0, " secs"

    call pgmtxt("t", -6.0, 0.0, 0.0, trim(string))
    write(string,'(a, i3)') 'Number of good tracking frames: ', nFrames
    call pgmtxt("t", -7.0, 0.0, 0.0, trim(string))

    write(string,'(a, i3)') 'Number of bad tracking frames: ', nBad
 
    call pgmtxt("t", -8.0, 0.0, 0.0, trim(string))
    if (exposureInfo%binbpe) then
        call pgmtxt("t", -9.0, 0.0, 0.0, "Data obtained in low-resolution mode")
    else
        call pgmtxt("t", -9.0, 0.0, 0.0, "Data obtained in high-resolution mode")
    end if
    
end subroutine PlotExposureDetails

end module omdrifthist_mod














